@@ -1,8 +1,10 @@
 # vi: ts=4 expandtab
 #
 #    Copyright (C) 2009-2010 Canonical Ltd.
+#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
 #
 #    Author: Scott Moser <scott.moser@canonical.com>
+#    Author: Juerg Haefliger <juerg.haefliger@hp.com>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3, as
@@ -21,7 +23,8 @@ from cloudinit.CloudConfig import per_always
 
 frequency = per_always
 
-def handle(name,cfg,cloud,log,args):
+
+def handle(_name, cfg, _cloud, _log, _args):
     if util.get_cfg_option_bool(cfg, "disable_ec2_metadata", False):
-        fwall="route add -host 169.254.169.254 reject"
+        fwall = "route add -host 169.254.169.254 reject"
         subprocess.call(fwall.split(' '))
