@@ -1,8 +1,10 @@
 # vi: ts=4 expandtab
 #
 #    Copyright (C) 2009-2010 Canonical Ltd.
+#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
 #
 #    Author: Scott Moser <scott.moser@canonical.com>
+#    Author: Juerg Haefliger <juerg.haefliger@hp.com>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3, as
@@ -15,10 +17,13 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-import cloudinit
-import cloudinit.util as util
+
+#import cloudinit
+#import cloudinit.util as util
 from cloudinit.CloudConfig import per_instance
 
 frequency = per_instance
-def handle(name,cfg,cloud,log,args):
-   print "hi"
+
+
+def handle(_name, _cfg, _cloud, _log, _args):
+    print "hi"
