@@ -1,8 +1,10 @@
 # vi: ts=4 expandtab
 #
 #    Copyright (C) 2011 Canonical Ltd.
+#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
 #
 #    Author: Scott Moser <scott.moser@canonical.com>
+#    Author: Juerg Haefliger <juerg.haefliger@hp.com>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3, as
@@ -15,14 +17,16 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import cloudinit.util as util
-from cloudinit.CloudConfig import per_once, per_always, per_instance
-from cloudinit import get_cpath, get_ipath_cur
+from cloudinit.CloudConfig import per_instance
+from cloudinit import get_ipath_cur
 
 frequency = per_instance
 runparts_path = "%s/%s" % (get_ipath_cur(), "scripts")
 
-def handle(name,cfg,cloud,log,args):
+
+def handle(_name, _cfg, _cloud, log, _args):
     try:
         util.runparts(runparts_path)
     except:
