@@ -1,8 +1,10 @@
 # vi: ts=4 expandtab
 #
 #    Copyright (C) 2009-2010 Canonical Ltd.
+#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
 #
 #    Author: Scott Moser <scott.moser@canonical.com>
+#    Author: Juerg Hafliger <juerg.haefliger@hp.com>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3, as
@@ -20,21 +22,23 @@
 DEP_FILESYSTEM = "FILESYSTEM"
 DEP_NETWORK = "NETWORK"
 
-import UserDataHandler as ud
+import cloudinit.UserDataHandler as ud
 import cloudinit.util as util
+import socket
+
 
 class DataSource:
     userdata = None
     metadata = None
     userdata_raw = None
     cfgname = ""
-    # system config (passed in from cloudinit, 
+    # system config (passed in from cloudinit,
     # cloud-config before input from the DataSource)
-    sys_cfg = { }
+    sys_cfg = {}
     # datasource config, the cloud-config['datasource']['__name__']
-    ds_cfg = { }  # datasource config
+    ds_cfg = {}  # datasource config
 
-    def __init__(self,sys_cfg=None):
+    def __init__(self, sys_cfg=None):
         if not self.cfgname:
             name = str(self.__class__).split(".")[-1]
             if name.startswith("DataSource"):
@@ -44,7 +48,7 @@ class DataSource:
             self.sys_cfg = sys_cfg
 
         self.ds_cfg = util.get_cfg_by_path(self.sys_cfg,
-                          ("datasource",self.cfgname),self.ds_cfg)
+                          ("datasource", self.cfgname), self.ds_cfg)
 
     def get_userdata(self):
         if self.userdata == None:
@@ -54,22 +58,29 @@ class DataSource:
     def get_userdata_raw(self):
         return(self.userdata_raw)
 
-
     # the data sources' config_obj is a cloud-config formated
     # object that came to it from ways other than cloud-config
     # because cloud-config content would be handled elsewhere
     def get_config_obj(self):
-        return({ })
+        return({})
 
     def get_public_ssh_keys(self):
         keys = []
-        if not self.metadata.has_key('public-keys'): return([])
-        for keyname, klist in self.metadata['public-keys'].items():
+        if 'public-keys' not in self.metadata:
+            return([])
+
+        if isinstance(self.metadata['public-keys'], str):
+            return(str(self.metadata['public-keys']).splitlines())
+
+        if isinstance(self.metadata['public-keys'], list):
+            return(self.metadata['public-keys'])
+
+        for _keyname, klist in self.metadata['public-keys'].items():
             # lp:506332 uec metadata service responds with
             # data that makes boto populate a string for 'klist' rather
             # than a list.
-            if isinstance(klist,str):
-                klist = [ klist ]
+            if isinstance(klist, str):
+                klist = [klist]
             for pkey in klist:
                 # there is an empty string at the end of the keylist, trim it
                 if pkey:
@@ -77,7 +88,7 @@ class DataSource:
 
         return(keys)
 
-    def device_name_to_device(self, name):
+    def device_name_to_device(self, _name):
         # translate a 'name' to a device
         # the primary function at this point is on ec2
         # to consult metadata service, that has
@@ -89,27 +100,57 @@ class DataSource:
         return('en_US.UTF-8')
 
     def get_local_mirror(self):
-        return('http://archive.ubuntu.com/ubuntu/')
+        return None
 
     def get_instance_id(self):
         if 'instance-id' not in self.metadata:
-            return "ubuntuhost"
+            return "iid-datasource"
         return(self.metadata['instance-id'])
 
-    def get_hostname(self):
+    def get_hostname(self, fqdn=False):
+        defdomain = "localdomain"
+        defhost = "localhost"
+
+        domain = defdomain
         if not 'local-hostname' in self.metadata:
-            return None
+
+            # this is somewhat questionable really.
+            # the cloud datasource was asked for a hostname
+            # and didn't have one. raising error might be more appropriate
+            # but instead, basically look up the existing hostname
+            toks = []
+
+            hostname = socket.gethostname()
+
+            fqdn = util.get_fqdn_from_hosts(hostname)
+
+            if fqdn and fqdn.find(".") > 0:
+                toks = str(fqdn).split(".")
+            elif hostname:
+                toks = [hostname, defdomain]
+            else:
+                toks = [defhost, defdomain]
+
+        else:
+            # if there is an ipv4 address in 'local-hostname', then
+            # make up a hostname (LP: #475354) in format ip-xx.xx.xx.xx
+            lhost = self.metadata['local-hostname']
+            if is_ipv4(lhost):
+                toks = "ip-%s" % lhost.replace(".", "-")
+            else:
+                toks = lhost.split(".")
+
+        if len(toks) > 1:
+            hostname = toks[0]
+            domain = '.'.join(toks[1:])
+        else:
+            hostname = toks[0]
+
+        if fqdn:
+            return "%s.%s" % (hostname, domain)
+        else:
+            return hostname
 
-        toks = self.metadata['local-hostname'].split('.')
-        # if there is an ipv4 address in 'local-hostname', then
-        # make up a hostname (LP: #475354)
-        if len(toks) == 4:
-            try:
-                r = filter(lambda x: int(x) < 256 and x > 0, toks)
-                if len(r) == 4:
-                    return("ip-%s" % '-'.join(r))
-            except: pass
-        return toks[0]
 
 # return a list of classes that have the same depends as 'depends'
 # iterate through cfg_list, loading "DataSourceCollections" modules
@@ -122,11 +163,14 @@ class DataSource:
 #   ie, pkglist=[ "foo", "" ]
 #     will first try to load foo.DataSource<item>
 #     then DataSource<item>
-def list_sources(cfg_list, depends, pkglist=[]):
+def list_sources(cfg_list, depends, pkglist=None):
+    if pkglist is None:
+        pkglist = []
     retlist = []
     for ds_coll in cfg_list:
         for pkg in pkglist:
-            if pkg: pkg="%s." % pkg
+            if pkg:
+                pkg = "%s." % pkg
             try:
                 mod = __import__("%sDataSource%s" % (pkg, ds_coll))
                 if pkg:
@@ -138,18 +182,33 @@ def list_sources(cfg_list, depends, pkglist=[]):
                 raise
     return(retlist)
 
+
 # depends is a list of dependencies (DEP_FILESYSTEM)
 # dslist is a list of 2 item lists
-# dslist = [ 
+# dslist = [
 #   ( class, ( depends-that-this-class-needs ) )
 # }
 # it returns a list of 'class' that matched these deps exactly
 # it is a helper function for DataSourceCollections
 def list_from_depends(depends, dslist):
-    retlist = [ ]
+    retlist = []
     depset = set(depends)
     for elem in dslist:
         (cls, deps) = elem
         if depset == set(deps):
             retlist.append(cls)
     return(retlist)
+
+
+def is_ipv4(instr):
+    """ determine if input string is a ipv4 address. return boolean"""
+    toks = instr.split('.')
+    if len(toks) != 4:
+        return False
+
+    try:
+        toks = [x for x in toks if (int(x) < 256 and int(x) > 0)]
+    except:
+        return False
+
+    return (len(toks) == 4)
